package com.ccwangluo.ccim.manage;


import com.ccwangluo.ccim.CCIMClient;
import com.ccwangluo.ccim.listener.OnConnectListener;
import com.ccwangluo.ccim.modle.IrequestData;
import com.ccwangluo.ccim.modle.MyIReaderProtocol;
import com.ccwangluo.ccim.listener.MessageListener;
import com.ccwangluo.ccim.listener.ResultEventCallback;
import com.ccwangluo.ccim.util.HmacUtil;
import com.xuhao.didi.core.pojo.OriginalData;
import com.xuhao.didi.socket.client.sdk.OkSocket;
import com.xuhao.didi.socket.client.sdk.client.ConnectionInfo;
import com.xuhao.didi.socket.client.sdk.client.OkSocketOptions;
import com.xuhao.didi.socket.client.sdk.client.action.SocketActionAdapter;
import com.xuhao.didi.socket.client.sdk.client.connection.IConnectionManager;
import java.util.logging.Logger;

import ccim.protocol.Channel;
import ccim.protocol.Chat;
import ccim.protocol.Common;
import ccim.protocol.Message;
import ccim.protocol.Push;

public class SocketManageer {
    private final Logger logger = Logger.getLogger(CCIMClient.class.getName());
    private static final String SERVER_IP = "117.144.156.2";
//    private static final String SERVER_IP = "192.168.31.216";
    private static final int SERVER_PORT = 8000;

    private static final int version = 1;

    private volatile int id = 1;

    private final CallBackManager<Chat.ChatResponseMessage> callBackManager;
    public SocketManageer(CallBackManager<Chat.ChatResponseMessage> callBackManager) {
        this.callBackManager = callBackManager;
    }

    private OnConnectListener onConnectListener;

    private ConnectionInfo chatInfo;


    public ConnectionInfo getChatInfo() {
        return chatInfo;
    }


    public void connectChatServer(String appKey, String userName,String extData, Long userId,Long ts, String sign) {
        Chat.ConnectRequest builder = Chat.ConnectRequest.newBuilder()
                .setUsername(userName)
                .setAppKey(appKey)
                .setUserId(userId)
                .setExtData(extData)
                .setTs(ts)
                .setSign(sign)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(1).setSeq(1).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder().setConnectReq(builder).setHeader(header).build();
        OkSocket.open(getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(idPlus(), new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                onConnectListener.onSocketConnection();
            }
        });
    }

    public void initChat(String appKey, String userName,String extData, Long userId, Long ts, String sign, OnConnectListener resultCallback) {
        onConnectListener = resultCallback;
        if (chatInfo != null){
            OkSocket.open(chatInfo).disconnect();
        }
        id = 1;
        this.chatInfo  = initServer(SERVER_IP, SERVER_PORT, new MessageListener() {
            @Override
            public void onServerConnect() {
//                if(id == 1) {
                    connectChatServer(appKey, userName,extData, userId, ts, sign);
//                }
            }

            @Override
            public void onMessageListener(OriginalData data) {
                try {
                    Chat.ChatResponseMessage chatResponseMessage = Chat.ChatResponseMessage.parseFrom(data.getBodyBytes());
                    logger.info(chatResponseMessage.toString());
                    int seq = chatResponseMessage.getHeader().getSeq();
                    ResultEventCallback<Chat.ChatResponseMessage> callBack = callBackManager.getCallBack(seq);
                    if (callBack != null) {
                        if (seq > 0) {
                            if (chatResponseMessage.getHeader().getCode() == Common.StatusCode.STATUS_CODE_OK) {
                                callBack.onSuccess(chatResponseMessage);
                            } else {
                                callBack.onFailed(chatResponseMessage.getHeader().getCode().getNumber());
                            }
                            callBackManager.removeCallBack(seq);
                        }
                        else if (seq == 0) { //push消息
                            callBack.onSuccess(chatResponseMessage);
                        }
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    private ConnectionInfo initServer(String ip, int port, MessageListener messageListener) {
        ConnectionInfo authInfo = new ConnectionInfo(ip, port);
        //调用OkSocket,开启这次连接的通道,拿到通道Manager
        IConnectionManager manager = OkSocket.open(authInfo);
        //获得当前连接通道的参配对象
        OkSocketOptions.Builder okOptionsBuilder = new OkSocketOptions.Builder();
        okOptionsBuilder.setReaderProtocol(new MyIReaderProtocol());
        //将新的修改后的参配设置给连接管理器
        manager.option(okOptionsBuilder.build());
        //调用通道进行连接
        manager.connect();

        manager.registerReceiver(new SocketActionAdapter() {
            @Override
            public void onSocketConnectionSuccess(ConnectionInfo info, String action) {
                super.onSocketConnectionSuccess(info, action);
                messageListener.onServerConnect();
            }

            @Override
            public void onSocketDisconnection(ConnectionInfo info, String action, Exception e) {
                super.onSocketDisconnection(info, action, e);
                if (onConnectListener != null) {
                    onConnectListener.onSocketDisconnection();
                }
            }

            @Override
            public void onSocketReadResponse(ConnectionInfo info, String action, OriginalData data) {
                super.onSocketReadResponse(info, action, data);
                messageListener.onMessageListener(data);
            }
        });
        return authInfo;
    }

    public int sendRequest(Object request) {
        int idPlus = idPlus();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(idPlus).build();
        Chat.ChatRequestMessage.Builder builder  = Chat.ChatRequestMessage.newBuilder();
        builder.setHeader(header);
        if (request instanceof Chat.PullContactsRequest){
            builder.setPullContactsReq((Chat.PullContactsRequest)request);
        }else if (request instanceof Chat.SearchRequest){
            builder.setSearchReq((Chat.SearchRequest)request);
        }else if (request instanceof Chat.AddContactRequest){
            builder.setAddContactReq((Chat.AddContactRequest)request);
        } else if (request instanceof Chat.ContactRequestsRequest){
            builder.setContactRequestsReq((Chat.ContactRequestsRequest) request);
        } else if(request instanceof Chat.CheckContactReqRequest){
            builder.setCheckContactReqReq((Chat.CheckContactReqRequest) request);
        } else if (request instanceof Chat.CreateContactGroupRequest){
            builder.setCreateContactGroupReq((Chat.CreateContactGroupRequest) request);
        } else if (request instanceof Chat.MoveContactGroupRequest){
            builder.setMoveContactGroupReq((Chat.MoveContactGroupRequest) request);
        } else if (request instanceof Chat.DeleteContactRequest){
            builder.setDeleteContactReq((Chat.DeleteContactRequest) request);
        } else if (request instanceof Chat.BlockContactRequest){
            builder.setBlockContactReq((Chat.BlockContactRequest) request);
        } else if (request instanceof Chat.ReportIllegalUserRequest){
            builder.setReportIllegalUserReq((Chat.ReportIllegalUserRequest) request);
        } else if (request instanceof Message.SendMessageRequest){
            builder.setSendMsgReq((Message.SendMessageRequest) request);
        } else if (request instanceof Message.SendChannelMessageRequest){
            builder.setSendChannelMsgReq((Message.SendChannelMessageRequest) request);
        } else if (request instanceof Push.PushPrivateMessageAckRequest){
            builder.setPushPrivateMsgAckReq((Push.PushPrivateMessageAckRequest) request);
        } else if (request instanceof Push.PushChannelMessageAckRequest){
            builder.setPushChannelMsgAckReq((Push.PushChannelMessageAckRequest) request);
        } else if (request instanceof Message.PullOfflineMessageRequest){
            builder.setPullOfflineMsgReq((Message.PullOfflineMessageRequest) request);
        } else if (request instanceof Channel.CreateChannelRequest){
            builder.setCreateChannelReq((Channel.CreateChannelRequest) request);
        } else if (request instanceof  Channel.JoinChannelRequest){
            builder.setJoinChannelReq((Channel.JoinChannelRequest) request);
        } else if (request instanceof Channel.RemoveMemberRequest){
            builder.setRemoveMemberReq((Channel.RemoveMemberRequest) request);
        } else if (request instanceof Channel.ExitChannelRequest){
            builder.setExitChannelReq((Channel.ExitChannelRequest) request);
        }

        OkSocket.open(getChatInfo())
                .send(new IrequestData(builder.build().toByteArray()));

        return idPlus;
    }

    public synchronized int idPlus() {
        if (id == Integer.MAX_VALUE){
            id = 1;
        }
        return id++;
    }

}
